@@ -309,7 +309,9 @@ gtk_clutter_embed_realize (GtkWidget *widget)
                         | GDK_KEY_RELEASE_MASK
                         | GDK_POINTER_MOTION_MASK
                         | GDK_ENTER_NOTIFY_MASK
-                        | GDK_LEAVE_NOTIFY_MASK;
+                        | GDK_LEAVE_NOTIFY_MASK
+                        | GDK_TOUCH_MASK
+                        | GDK_SMOOTH_SCROLL_MASK;
 
   attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
 
@@ -497,6 +499,7 @@ gtk_clutter_embed_size_allocate (GtkWidget     *widget,
                                  GtkAllocation *allocation)
 {
   GtkClutterEmbedPrivate *priv = GTK_CLUTTER_EMBED (widget)->priv;
+  int scale_factor = gtk_widget_get_scale_factor (widget);
 
   gtk_widget_set_allocation (widget, allocation);
 
@@ -523,8 +526,8 @@ gtk_clutter_embed_size_allocate (GtkWidget     *widget,
 	{
 	  XConfigureEvent xevent = { ConfigureNotify };
 	  xevent.window = GDK_WINDOW_XID (gtk_widget_get_window (widget));
-	  xevent.width = allocation->width;
-	  xevent.height = allocation->height;
+	  xevent.width = allocation->width * scale_factor;
+	  xevent.height = allocation->height * scale_factor;
 
 	  /* Ensure cogl knows about the new size immediately, as we will
 	     draw before we get the ConfigureNotify response. */
