@@ -3,17 +3,6 @@
 
 #include <clutter-gtk/clutter-gtk.h>
 
-static void
-on_stage2_allocation_changed (ClutterActor           *stage_2,
-                              const ClutterActorBox  *allocation,
-                              ClutterAllocationFlags  flags,
-                              ClutterActor           *texture_2)
-{
-  clutter_actor_set_position (texture_2,
-                              (allocation->x2 - allocation->x1) / 2,
-                              (allocation->y2 - allocation->y1) / 2);
-}
-
 int
 main (int argc, char *argv[])
 {
@@ -55,10 +44,9 @@ main (int argc, char *argv[])
                                       GTK_STOCK_DIALOG_INFO,
                                       GTK_ICON_SIZE_DIALOG,
                                       NULL);
-  clutter_actor_set_anchor_point (tex1,
-                                  clutter_actor_get_width (tex1) / 2,
-                                  clutter_actor_get_height (tex1) / 2);
-  clutter_actor_set_position (tex1, 160, 120);
+  clutter_actor_set_position (tex1,
+                              160 - clutter_actor_get_width (tex1) / 2.0,
+                              120 - clutter_actor_get_height (tex1) / 2.0);
   clutter_actor_add_child (stage1, tex1); 
   clutter_actor_show (tex1);
 
@@ -74,18 +62,11 @@ main (int argc, char *argv[])
                                           "user-info",
                                           GTK_ICON_SIZE_BUTTON,
                                           NULL);
-  clutter_actor_set_anchor_point (tex2,
-                                  clutter_actor_get_width (tex2) / 2,
-                                  clutter_actor_get_height (tex2) / 2);
-  clutter_actor_set_position (tex2, 160, 60);
+  clutter_actor_add_constraint (tex2, clutter_align_constraint_new (stage2, CLUTTER_ALIGN_BOTH, .5));
   clutter_actor_add_child (stage2, tex2);
 
   gtk_container_add (GTK_CONTAINER (vbox), clutter2);
 
-  g_signal_connect (stage2, "allocation-changed",
-                    G_CALLBACK (on_stage2_allocation_changed),
-                    tex2);
-
   gtk_widget_show_all (window);
 
   gtk_main();
